#!/usr/bin/env python
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Grec <jgrec@redhat.com>
# Authors: Michal Minar <miminar@redhat.com>
#
"""
Miscellaneous unit tests for OpenLMI Software provider.
"""

import os
import subprocess
import unittest

import swbase

BROKER = os.environ.get("LMI_CIMOM_BROKER", "tog-pegasus")
OPENLMI_SOFTWARE_PKG_NAME = 'openlmi-software'

class TestSoftwareProvider(swbase.SwTestCase):
    """
    Tests not related to particular class of OpenLMI Software profiles.
    """

    def _is_provider_ready(self):
        """
        Test whether software provider anwers requests.
        """
        service = self.ns.LMI_SoftwareInstallationService.first_instance()
        self.assertNotEqual(service, None,
                "software provider is up and running")
        insts = service.FindIdentity(Name=service, ExactMatch=True)
        self.assertGreater(len(insts), 0,
                "software provider correctly find kernel package")

    @unittest.skip("rhbz#1026270")
    def test_reinstall_broker_by_yum(self):
        """
        Test reinstallation of current broker and check everything still works.

        Get broker from BROKER global variable.
        Check afterwards that provider handles requests.
        """
        self.assertIn(BROKER, ["tog-pegasus", "sblim-sfcb"], "broker is known")
        subprocess.call(["/usr/bin/yum", "-y", "reinstall", BROKER])
        self._is_provider_ready()

    def test_reinstall_software_provider_by_yum(self):
        """
        Test reinstallation of ``openlmi-software`` package which
        provides software provider which is being tested.

        Check afterwards that provider handles requests.
        """
        subprocess.call(["/usr/bin/yum", "-y", "reinstall", "openlmi-software"])
        self._is_provider_ready()




# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Michal Minar <miminar@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for SoftwareConfiguration class.

SoftwareConfiguration
---------------------

.. autoclass:: SoftwareConfiguration
    :members:

"""

from lmi.base.BaseConfiguration import BaseConfiguration

class SoftwareConfiguration(BaseConfiguration):
    """
        Configuration class specific to software providers.
        OpenLMI configuration file should reside in
        /etc/openlmi/software/software.conf.
    """

    @classmethod
    def provider_prefix(cls):
        return "software"

    @classmethod
    def default_options(cls):
        """ :rtype: (``dict``) Dictionary of default values. """
        defaults = BaseConfiguration.default_options().copy()
        # Maximum time in seconds to wait for a job to accomplish.
        # If timeout expires, spawned process is checked (it might
        # be possibly killed) and is respawned in case it's dead.
        defaults["WaitCompleteTimeout" ] = "30"
        # Number of seconds to wait before next try to lock yum package
        # database. This applies, when yum database is locked by another
        # process.
        defaults["LockWaitInterval"] = "0.5"
        # Number of seconds to keep package cache in memory after the last use
        # (caused by user request). Package cache takes up a lot of memory.
        defaults["FreeDatabaseTimeout"] = "60"
        # Minimum time to keep asynchronous job in cache after completion. In
        # seconds.
        defaults["MinimumTimeBeforeRemoval"] = "10"
        # Number of seconds to keep asynchronous job after its completion.
        # It applies only to jobs with DeleteOnCompletion == True.
        defaults["DefaultTimeBeforeRemoval"] = "300"
        # Default asynchronous job priority.
        defaults["DefaultPriority"] = "10"
        return defaults

    @classmethod
    def mandatory_sections(cls):
        return list(BaseConfiguration.mandatory_sections()) + [
            'Yum', 'Jobs', 'YumWorkerLog']




# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Michal Minar <miminar@redhat.com>
#
"""
Common utilities for OpenLMI python providers.
"""

import socket

def parse_instance_id(instance_id, classname=None):
    """
    Parse InstanceID, check it has LMI:<classname>:<ID> format and return
    the ID. Return None if the format is bad.
    :param instance_id: (``string``) String to parse.
    :param classname: (``string``) Name of class, whose InstanceID we parse.
        If the classname is None, it won't be checked.
    :returns: ``string`` with the ID.
    """
    parts = instance_id.split(":", 2)
    if len(parts) != 3:
        return None
    if parts[0] != "LMI":
        return None
    real_classname = parts[1]
    if classname and real_classname.lower() != classname.lower():
        return None
    return parts[2]

def is_this_system(system_name):
    """
    Return ``True`` if given *system_name* matches the hostname of currently
    running system.

    Global configuration object must be initialized before calling this
    function.

    :rtype: boolean
    """
    return (  socket.gethostbyaddr(system_name)[0]
           == socket.gethostbyaddr(socket.gethostname())[0])



# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Michal Minar <miminar@redhat.com>
#
"""
Base class and utilities for test suits written upon lmi shell.
"""

import functools
import inspect
import pywbem

from lmi.test import base
from lmi.shell import connect
from lmi.shell import LMIInstance
from lmi.shell import LMIInstanceName
from lmi.shell import LMIUtil

def enable_lmi_exceptions(method):
    """
    Function or method decorator enabling exceptions to be raised from under
    lmi shell intestines.
    """
    @functools.wraps(method)
    def _wrapper(*args, **kwargs):
        """ Enable exceptions in wrapped function. """
        original = LMIUtil.lmi_get_use_exceptions()
        LMIUtil.lmi_set_use_exceptions(True)
        try:
            retval = method(*args, **kwargs)
        finally:
            LMIUtil.lmi_set_use_exceptions(original)
        return retval

    return _wrapper

def to_cim_object(obj):
    """
    :returns: Wrapped object of from inside of shell abstractions.
    """
    if isinstance(obj, (LMIInstance, LMIInstanceName)):
        return obj.wrapped_object
    return obj

class LmiTestCase(base.BaseLmiTestCase):
    """
    Base class for all LMI test cases based on lmi shell.
    """

    #: Once you override this in subclass with a name of CIM class to be tested,
    #: you can use :py:attr:`LmiTestCase.cim_class` to get reference to a shell
    #: wrapper of this class.
    CLASS_NAME = None

    _SYSTEM_INAME = None

    @classmethod
    def setUpClass(cls):
        base.BaseLmiTestCase.setUpClass.im_func(cls)

    @property
    def conn(self):
        """
        :returns: Active connection to *CIMOM* wrapped by *lmi shell*
            abstraction.
        :rtype: :py:class:`lmi.shell.LMIConnection`
        """
        if not hasattr(self, '_shellconnection'):
            kwargs = {}
            con_argspec = inspect.getargspec(connect)
            # support older versions of lmi shell
            if 'verify_server_cert' in con_argspec.args or con_argspec.keywords:
                # newer name
                kwargs['verify_server_cert'] = False
            elif 'verify_certificate' in con_argspec.args:
                # older one
                kwargs['verify_certificate'] = False
            self._shellconnection = connect(
                    self.url, self.username, self.password, **kwargs)
        return self._shellconnection

    @property
    def ns(self):
        """
        :returns: Namespace object representing CIM ``"root/cimv2"`` namespace.
        :rtype: :py:class:`lmi.shell.LMINamespace`
        """
        return self.conn.root.cimv2

    @property
    def cim_class(self):
        """
        A convenience accessor to ``self.conn.root.cimv2.<CLASS_NAME>``.
        You need to override ``CLASS_NAME`` attribute of this class in order
        to use this.

        :returns: Lmi shell wrapper of CIM class to be tested.
        :rtype: :py:class:`lmi.shell.LMIClass`
        """
        if self.CLASS_NAME is None:
            return None
        return getattr(self.ns, self.CLASS_NAME)

    @property
    def system_iname(self):
        """
        :returns: Instance of ``CIM_ComputerSystem`` registered with *CIMOM*.
        :rtype: :py:class:`lmi.shell.LMIInstanceName`
        """
        if LmiTestCase._SYSTEM_INAME is None:
            LmiTestCase._SYSTEM_INAME = getattr(self.ns, self.system_cs_name) \
                    .first_instance_name()
        return LmiTestCase._SYSTEM_INAME.copy()

    def assertCIMIsSubclass(self, cls, base_cls):
        """
        Checks, whether cls is subclass of base_cls from CIM perspective.
        @param cls name of subclass
        @param base_cls name of base class
        """
        if not isinstance(cls, basestring):
            raise TypeError("cls must be a string")
        if not isinstance(base_cls, basestring):
            raise TypeError("base_cls must be a string")
        return self.assertTrue(pywbem.is_subclass(self.conn._client._cliconn,
            "root/cimv2", base_cls, cls))

    def assertRaisesCIM(self, cim_err_code, func, *args, **kwds):
        """
        This test passes if given function called with supplied arguments
        raises :py:class:`pywbem.CIMError` with given cim error code.
        """
        base.BaseLmiTestCase.assertRaisesCIM(
                self, cim_err_code, func, *args, **kwds)

    def assertCIMNameEqual(self, fst, snd, msg=None):
        """
        Compare two objects of :py:class:`pywbem.CIMInstanceName`. Their host
        properties are not checked.
        """
        base.BaseLmiTestCase.assertCIMNameEqual(
                self,
                to_cim_object(fst),
                to_cim_object(snd),
                msg)

    def assertCIMNameIn(self, name, candidates):
        """
        Checks that given :py:class:`pywbem.CIMInstanceName` is present in
        set of candidates. It compares all properties but ``host``.
        """
        name = to_cim_object(name)
        candidates = [to_cim_object(c) for c in candidates]
        base.BaseLmiTestCase.assertCIMNameIn(self, name, candidates)




#!/usr/bin/env python
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Michal Minar <miminar@redhat.com>
#
"""
Unit tests for ``LMI_SoftwareIdentity`` provider.
"""

from datetime import datetime, timedelta
import pywbem
import unittest

from lmi.test.lmibase import enable_lmi_exceptions
import package
import swbase
import util

class TestSoftwareIdentity(swbase.SwTestCase):
    """
    Basic cim operations test on ``LMI_SoftwareIdentity``.
    """

    CLASS_NAME = "LMI_SoftwareIdentity"
    KEYS = ("InstanceID", )

    def make_op(self, pkg):
        """
        :returns: Object path of ``LMI_SoftwareIdentity``
        :rtype: :py:class:`lmi.shell.LMIInstanceName`
        """
        return util.make_pkg_op(self.ns, pkg)

    def _check_package_instance(self, pkg, inst):
        """
        Check properties of ``LMI_SoftwareIdentity`` instance and its
        correspondence to represented *pkg*.
        """
        self.assertNotEqual(inst, None,
                "instance must exist for package %s" % pkg)
        objpath = self.make_op(pkg)
        self.assertCIMNameEqual(objpath, inst.path)
        for key in self.KEYS:
            self.assertEqual(getattr(inst, key), getattr(inst.path, key))
        self.assertEqual(set(inst.path.key_properties()), set(self.KEYS))
        self.assertEqual(pkg.name, inst.Name)

        self.assertEqual(inst.InstanceID, 'LMI:LMI_SoftwareIdentity:'
                + pkg.nevra)
        self.assertEqual(inst.Caption, pkg.summary)
        self.assertIsInstance(inst.Description, basestring)
        self.assertEqual(inst.VersionString, pkg.evra,
                "VersionString does not match evra for pkg %s" % pkg)
        self.assertTrue(inst.IsEntity)
        self.assertEqual(inst.Name, pkg.name,
                "Name does not match for pkg %s" % pkg)
        self.assertIsInstance(inst.Epoch, pywbem.Uint32,
                "Epoch does not match for pkg %s" % pkg)
        self.assertEqual(inst.Epoch, pkg.epoch)
        self.assertEqual(inst.Version, pkg.ver,
                "Version does not match for pkg %s" % pkg)
        self.assertEqual(inst.Release, pkg.rel,
                "Release does not match for pkg %s" % pkg)
        self.assertEqual(inst.Architecture, pkg.arch,
                "Architecture does not match for pkg %s" % pkg)
        self.assertEqual(inst.ElementName, pkg.nevra,
                "ElementName must match nevra of pkg %s" % pkg)
        self.assertTrue(inst.InstanceID.endswith(inst.ElementName),
                "InstanceID has package nevra at its end for pkg %s" % pkg)

        if package.is_pkg_installed(pkg):
            self.assertNotEqual(inst.InstallDate, None)
        else:
            self.assertEqual(inst.InstallDate, None)

    @swbase.test_with_packages(**{'stable#pkg1' : False})
    def test_get_instance_installed(self):
        """
        Test ``GetInstance()`` call on ``LMI_SoftwareIdentity`` with
        installed package.
        """
        pkg = self.get_repo('stable')['pkg1']
        package.remove_pkgs(pkg.name)
        package.install_pkgs(pkg)
        self.assertTrue(package.is_pkg_installed(pkg))

        objpath = self.make_op(pkg)
        inst = objpath.to_instance()
        self.assertNotEqual(inst, None, "GetInstance is successful on %s" % pkg)
        self.assertIsNotNone(inst.InstallDate)
        time_stamp = datetime.now(pywbem.cim_types.MinutesFromUTC(0)) \
                - timedelta(seconds=5)
        self.assertGreater(inst.InstallDate.datetime, time_stamp)
        self._check_package_instance(pkg, inst)

    @swbase.test_with_repos('stable')
    @swbase.test_with_packages(**{'stable#pkg2' : False})
    def test_get_instance_not_installed(self):
        """
        Test ``GetInstance()`` call on ``LMI_SoftwareIdentity`` with not
        installed package.
        """
        pkg = self.get_repo('stable')['pkg2']
        self.assertFalse(package.is_pkg_installed(pkg))

        self._check_package_instance(pkg, self.make_op(pkg).to_instance())

    @swbase.test_with_packages('stable#pkg1', 'stable#pkg4')
    def test_get_instance_without_epoch(self):
        """
        Test ``GetInstance()`` on ``LMI_SoftwareIdentity`` with epoch part
        omitted.
        """
        pkg = self.get_repo('stable')['pkg1']
        objpath = self.make_op(pkg)
        objpath.wrapped_object['InstanceID'] = 'LMI:LMI_SoftwareIdentity:' \
                + pkg.get_nevra('NEVER')
        self.assertFalse('0:' in objpath.InstanceID)
        inst = objpath.to_instance()
        self.assertNotEqual(inst, None)
        self.assertTrue('0:' in pkg.nevra)
        self.assertTrue(inst.InstanceID.endswith(pkg.nevra))

    @swbase.test_with_repos('stable')
    @swbase.test_with_packages(**{'stable#pkg3' : False})
    def test_refresh_after_install(self):
        """
        Test instance refresh of ``LMI_SoftwareIdentity`` after package
        installation.
        """
        pkg = self.get_repo('stable')['pkg3']
        self.assertFalse(package.is_pkg_installed(pkg))
        inst = self.make_op(pkg).to_instance()
        self.assertEqual(inst.InstallDate, None)
        self._check_package_instance(pkg, inst)

        # install it
        package.install_pkgs(pkg)
        self.assertTrue(package.is_pkg_installed(pkg))
        inst.refresh()
        self.assertNotEqual(inst.InstallDate, None)
        self._check_package_instance(pkg, inst)

    @swbase.test_with_repos('stable')
    @swbase.test_with_packages('stable#pkg4')
    def test_refresh_after_uninstall(self):
        """
        Test instance refresh of ``LMI_SoftwareIdentity`` after package
        uninstallation.
        """
        pkg = self.get_repo('stable')['pkg4']
        self.assertTrue(package.is_pkg_installed(pkg))
        inst = self.make_op(pkg).to_instance()
        self.assertNotEqual(inst.InstallDate, None)
        self._check_package_instance(pkg, inst)

        # install it
        package.remove_pkgs(pkg.name)
        self.assertFalse(package.is_pkg_installed(pkg))
        inst.refresh()
        self.assertEqual(inst.InstallDate, None)
        self._check_package_instance(pkg, inst)

    @enable_lmi_exceptions
    @swbase.test_with_packages('stable#pkg3')
    def test_get_instance_invalid(self):
        """
        Test ``GetInstance()`` call on invalid object path.
        """
        pkg = self.get_repo('stable')['pkg3']
        # leave out arch part
        nvr = '%s-%s-%s' % (pkg.name, pkg.ver, pkg.rel)
        objpath = self.make_op(nvr)
        self.assertRaisesCIM(pywbem.CIM_ERR_NOT_FOUND, objpath.to_instance)

        # leave out version part
        nra = '%s-%s.%s' % (pkg.name, pkg.rel, pkg.arch)
        objpath = self.make_op(nra)
        self.assertRaisesCIM(pywbem.CIM_ERR_NOT_FOUND, objpath.to_instance)

        # leave out release part
        nva = '%s-%s.%s' % (pkg.name, pkg.ver, pkg.arch)
        objpath = self.make_op(nva)
        self.assertRaisesCIM(pywbem.CIM_ERR_NOT_FOUND, objpath.to_instance)

        # leave out name part
        vra = '%s-%s.%s' % (pkg.ver, pkg.rel, pkg.arch)
        objpath = self.make_op(vra)
        self.assertRaisesCIM(pywbem.CIM_ERR_INVALID_PARAMETER,
                objpath.to_instance)

        # leave out just epoch
        nvra = '%s-%s-%s.%s' % (pkg.name, pkg.ver, pkg.rel, pkg.arch)
        objpath = self.make_op(nvra)
        inst = objpath.to_instance()
        self.assertNotEqual(inst, None)

    @swbase.test_with_packages('misc#funny-version', 'misc#funny-release')
    def test_get_instance_funny(self):
        """
        Test ``GetInstance()`` on package with quite odd nevra.
        """
        pkg = self.get_repo('misc')['funny-version']
        self.assertTrue(package.is_pkg_installed(pkg))
        objpath = self.make_op(pkg)
        inst = objpath.to_instance()
        self.assertNotEqual(inst, None, "GetInstance is successful on %s" % pkg)
        self._check_package_instance(pkg, inst)

        pkg = self.get_repo('misc')['funny-release']
        self.assertTrue(package.is_pkg_installed(pkg))
        objpath = self.make_op(pkg)
        inst = objpath.to_instance()
        self.assertNotEqual(inst, None, "GetInstance is successful on %s" % pkg)
        self._check_package_instance(pkg, inst)

    @enable_lmi_exceptions
    def test_enum_instance_names(self):
        """
        Test ``EnumInstanceNames()`` call on ``LMI_SoftwareIdentity``.
        """
        self.assertRaisesCIM(pywbem.CIM_ERR_NOT_SUPPORTED,
                self.cim_class.instance_names)

def suite():
    """For unittest loaders."""
    return unittest.TestLoader().loadTestsFromTestCase(
            TestSoftwareIdentity)

if __name__ == '__main__':
    unittest.main()



# -*- encoding: utf-8 -*-
# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

"""
Just a common functionality related to AffectedSoftwareJobElement provider.
"""

import pywbem

from lmi.providers import cmpi_logging
from lmi.providers import ComputerSystem
from lmi.software import util
from lmi.software.core import Identity
from lmi.software.core import IdentityFileCheck
from lmi.software.core import Job
from lmi.software.core import SystemCollection
from lmi.software.yumdb import jobs
from lmi.software.yumdb import PackageInfo

LOG = cmpi_logging.get_logger(__name__)

class Values(object):
    class ElementEffects(object):
        Unknown = pywbem.Uint16(0)
        Other = pywbem.Uint16(1)
        Exclusive_Use = pywbem.Uint16(2)
        Performance_Impact = pywbem.Uint16(3)
        Element_Integrity = pywbem.Uint16(4)
        Create = pywbem.Uint16(5)
        _reverse_map = {
                0: 'Unknown',
                1: 'Other',
                2: 'Exclusive Use',
                3: 'Performance Impact',
                4: 'Element Integrity',
                5: 'Create'
        }

@cmpi_logging.trace_function
def check_path(env, op):
    """
    Checks, whether object path is valid.

    Return internal object representing job and object path of affected element
    as a pair: ``(job, affected)``.
    """
    if not isinstance(op, pywbem.CIMInstanceName):
        raise TypeError("op must be a CIMInstanceName")
    ch = env.get_cimom_handle()

    job = op['AffectingElement'] = Job.object_path2job(op['AffectingElement'])
    affected = op['AffectedElement']
    if ch.is_subclass(affected.namespace, sub=affected.classname,
            super='LMI_SoftwareIdentity'):
        pkg_info = Identity.object_path2pkg(affected, kind='all')
        if isinstance(job, jobs.YumSpecificPackageJob):
            if isinstance(job.pkg, PackageInfo):
                if pkg_info != job.pkg:
                    raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                            "AffectedElement does not match job's package:"
                            " \"%s\" != \"%s\"." % (pkg_info, job.pkg))
            else:
                flt = pkg_info.key_props
                flt.pop('repoid', None)
                if util.nevra2filter(job.pkg) != flt:
                    raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                            "AffectedElement does not match job's package:"
                            " \"%s\" != \"%s\"." % (pkg_info, job.pkg))
            affected = Identity.pkg2model(pkg_info)
        elif isinstance(job, jobs.YumInstallPackageFromURI):
            if job.state == job.COMPLETED:
                affected = Identity.pkg2model(job.result_data)
            else:
                # TODO: this should be somehow obtained from downloaded
                # package before installation
                raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                        "No SoftwareIdentity is associated to given job.")
        else:
            LOG().error("Unsupported async job: %s", job)
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "No associated SoftwareIdentity.")
    elif ch.is_subclass(affected.namespace, sub=affected.classname,
            super='LMI_SystemSoftwareCollection'):
        SystemCollection.check_path(env, affected, "AffectedElement")
        affected = SystemCollection.get_path()
    elif ch.is_subclass(affected.namespace, sub=affected.classname,
            super="CIM_ComputerSystem"):
        ComputerSystem.check_path(env, affected, "AffectedElement")
        affected = ComputerSystem.get_path(env)
    elif ch.is_subclass(affected.namespace, sub=affected.classname,
            super='LMI_SoftwareIdentityFileCheck'):
        if not isinstance(job, jobs.YumCheckPackage):
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Job must point to verification job, not to: \"%s\"" % job)
        if job.state != job.COMPLETED:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Associations to failed file checks for verification job"
                    " \"%s\" are not yet known, since job has not completed"
                    " yet." % op["AffectingElement"]["InstanceID"])
        pkg_info, _ = job.result_data
        file_check = IdentityFileCheck.object_path2file_check(affected)
        if file_check.pkg_info.nevra != pkg_info.nevra:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Affected element associated to job for another package:"
                    " \"%s\" != \"%s\"." % (file_check.pkg_info, pkg_info))
        if IdentityFileCheck.file_check_passed(file_check):
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Given file check reference passed the verification.")
    else:
        raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                "Expected an instance of LMI_SoftwareIdentity,"
                " LMI_SystemSoftwareCollection or CIM_ComputerSystem.")
    return (job, affected)

@cmpi_logging.trace_function
def job2affected_software_identity(job):
    """
    @return (path of SoftwareIdentity, ElementEffects array,
        OtherElementEffectsDescriptions array)
    """
    effects = [Values.ElementEffects.Other]
    descriptions = []
    if isinstance(job, jobs.YumSpecificPackageJob):
        if job.state == job.COMPLETED and job.result_data:
            if isinstance(job, jobs.YumCheckPackage):
                # get the first item out of (pkg_info, pkg_check)
                affected = Identity.pkg2model(job.result_data[0])
            else:
                affected = Identity.pkg2model(job.result_data)
        else:
            affected = Identity.pkg2model(job.pkg)
        if isinstance(job, jobs.YumInstallPackage):
            descriptions.append("Installing")
        elif isinstance(job, jobs.YumRemovePackage):
            descriptions.append("Removing")
        elif isinstance(job,
                (jobs.YumUpdatePackage, jobs.YumUpdateToPackage)):
            descriptions.append("Updating")
        elif isinstance(job, jobs.YumCheckPackage):
            descriptions.append("Verifying")
        else:
            descriptions.append("Modifying")
            LOG().error("Unhandled job: %s", job)
    elif isinstance(job, jobs.YumInstallPackageFromURI):
        if job.state == job.COMPLETED:
            affected = Identity.pkg2model(job.result_data)
            descriptions.append("Installing")
        else:
            # TODO: this should be somehow obtained from from downloaded
            # package before installation
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "No SoftwareIdentity is associated to given job.")
    else:
        LOG().error("Unsupported async job: %s", job)
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                "No associated SoftwareIdentity.")
    return (affected, effects, descriptions)

@cmpi_logging.trace_function
def fill_model_computer_system(env, model, job, keys_only=True):
    """
    Fills model's AffectedElement and all non-key properties.
    """
    model["AffectedElement"] = ComputerSystem.get_path(env)
    if not keys_only:
        model["ElementEffects"] = [Values.ElementEffects.Other]
        description = "Modifying software collection."
        if isinstance(job, (jobs.YumInstallPackage,
                jobs.YumInstallPackageFromURI)):
            description = "Installing software package to collection."
        elif isinstance(job, jobs.YumRemovePackage):
            description = "Removing package from software collection."
        elif isinstance(job, (jobs.YumUpdatePackage, jobs.YumUpdateToPackage)):
            description = "Updating software package."
        model["OtherElementEffectsDescriptions"] = [description]
    return model

@cmpi_logging.trace_function
def fill_model_system_collection(model, keys_only=True):
    """
    Fills model's AffectedElement and all non-key properties.
    """
    model["AffectedElement"] = SystemCollection.get_path()
    if not keys_only:
        model["ElementEffects"] = [Values.ElementEffects.Exclusive_Use]
        model["OtherElementEffectsDescriptions"] = [
                "Package database is locked."
        ]
    return model

@cmpi_logging.trace_function
def fill_model_failed_check(model, failed_check, keys_only=True):
    """
    Fills model's AffectedElement and all non-key properties.

    :param failed_check: (``CIMInstanceName``) Is on object path of failed
        file check.
    """
    if not isinstance(failed_check, pywbem.CIMInstanceName):
        raise TypeError("failed_check must be a CIMInstanceName")
    model["AffectedElement"] = failed_check
    if not keys_only:
        model["ElementEffects"] = [Values.ElementEffects.Other]
        model["OtherElementEffectsDescriptions"] = [
                "File did not pass the verification."
        ]
    return model

@cmpi_logging.trace_function
def generate_failed_checks(model, job, keys_only=True):
    """
    Generates associations between LMI_SoftwareVerificationJob and
    LMI_SoftwareIdentityFileCheck for files that did not pass the check.
    """
    out_params = Job.get_verification_out_params(job)
    if not "Failed" in out_params:
        return
    for failed in out_params["Failed"].value:
        yield fill_model_failed_check(model, failed, keys_only)

@cmpi_logging.trace_function
def generate_models_from_job(env, job, keys_only=True, model=None):
    """
    Generates all associations between job and affected elements.
    """
    if not isinstance(job, jobs.YumJob):
        raise TypeError("pkg must be an instance of PackageInfo or nevra")
    if model is None:
        model = util.new_instance_name("LMI_AffectedSoftwareJobElement")
        if not keys_only:
            model = pywbem.CIMInstance("LMI_AffectedSoftwareJobElement",
                    path=model)
    model["AffectingElement"] = Job.job2model(job)
    (si, element_effects, element_effects_descriptions) = \
            job2affected_software_identity(job)
    model["AffectedElement"] = si
    if not keys_only:
        model["ElementEffects"] = element_effects
        model["OtherElementEffectsDescriptions"] = \
                element_effects_descriptions
    yield model
    if not isinstance(job, jobs.YumCheckPackage):
        fill_model_system_collection(model, keys_only=keys_only)
        yield model
    else:   # package verification - associate to failed file checks
        for model in generate_failed_checks(model, job, keys_only=keys_only):
            yield model
    fill_model_computer_system(env, model, job, keys_only=keys_only)
    yield model




# -*- encoding: utf-8 -*-
# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Michal Minar <miminar@redhat.com>
#
"""
Module holding an abstraction for YUM repository.
"""

from datetime import datetime
import logging
import yum
import yum.Errors

# maps names of Repository properties to their corresponding property
# names in YumRepository object
PROPERTY_NAME_MAP = {
    "repoid"         : "id",
    "base_urls"      : "baseurl",
    "config_file"    : "repofile",
    "cost"           : "cost",
    "enabled"        : "enabled",
    "gpg_check"      : "gpgcheck",
    "last_edit"      : "repo_config_age",
    "metalink"       : "metalink",
    "mirror_list"    : "mirrorlist",
    "mirror_urls"    : "mirrorurls",
    "name"           : "name",
    "pkg_dir"        : "pkgdir",
    "ready"          : "ready",
    "repo_gpg_check" : "repo_gpgcheck",
    "timeout"        : "timeout"
}

def get_prop_from_yum_repo(repo, prop_name):
    """
    @param prop_name is one Repository properties
    @return value of property from object of YumRepository
    """
    if not isinstance(repo, yum.yumRepo.YumRepository):
        raise TypeError("repo must be in instance of yum.yumRepo.YumRepository")
    if prop_name in PROPERTY_NAME_MAP:
        val = getattr(repo, PROPERTY_NAME_MAP[prop_name])
        if prop_name == "last_edit":
            val = datetime.fromtimestamp(val)
        elif prop_name == "mirror_urls" and not repo.mirrorlist:
            val = None
        elif prop_name == "ready":
            val = val()
    elif prop_name in {"arch", "basearch", "releasever"}:
        val = repo.yumvar[prop_name]
    elif prop_name in {"revision", "last_update"}:
        if repo.enabled and repo.repoXML:
            md = repo.repoXML
            if prop_name == "last_update":
                val = datetime.fromtimestamp(md.timestamp)
            elif prop_name == "revision":
                val = long(md.revision)
            else:
                val = getattr(repo.repoXML, prop_name)
        else:
            val = None
    elif prop_name == "pkg_count":
        # this needs populated sack: ydb.repos.populateSack(repo.id)
        val = len(repo.sack)
    else:
        raise ValueError('Unknown repository property: "%s"' % prop_name)
    return val

class Repository(object):
    """
    Container for repository metadata. It represents yum repository.
    It's supposed to be passed from YumWorker to YumDB client and
    vice-versa.
    """
    __slots__ = (
          "objid",             # [int] id of python object on server process
          "repoid",               # [string] repository id name
                                # (name of config file)

          "arch",               # [string] architecture of packages
          "basearch",           # [string] base system architecture
          "base_urls",          # [list] base urls as strings
          #"cache",
          #"cache_dir",
          "name",            # [string] repository descriptive name
          "config_file",        # [string] file path to corresponding
                                # config file
          "cost",               # [int] cost of repository
          "enabled",            # [boolean] whether repo is enabled
          "gpg_check",          # [boolean] whether to check gpg signature
          #"metadata_expire",    # how long are metadata considered valid
          "last_edit",          # datetime of last config modification
          "last_update",        # datetime of last change of repository
                                # on server
          "metalink",           # Specifies a URL to a metalink file for the
            # repomd.xml, a list of mirrors for the entire repository are
            # generated by converting the mirrors for the repomd.xml file to a
            # baseurl.
          "mirror_list",        # url of mirrorlist, or None
          "mirror_urls",        # list of urls obtained from mirrorlist or None
          #"persist_dir",       #
          #"pkg_count",          # number of packages in directory
          "pkg_dir",            # directory with packages
          #"proxy",              # boolean saying whether this is a proxy
          "ready",              # boolean saying, whether it's ready for use
          "releasever",         # version of targeted distribution
          "repo_gpg_check",     # [boolean] whether to check gpg
                                # signarues of data
          "revision",
          "timeout",            # timeout for requests
    )

    def __init__(self, objid, repoid, arch, basearch, base_urls,
            config_file, cost, enabled, gpg_check, last_edit, last_update,
            name, pkg_dir, ready, releasever, repo_gpg_check, revision,
            timeout, metalink=None, mirror_list=None, mirror_urls=None):
        for arg in ('last_edit', 'last_update'):
            if (   locals()[arg] is not None
               and not isinstance(locals()[arg], datetime)):
                raise TypeError("%s must be an instance of datetime" % arg)
        if not isinstance(timeout, float):
            raise TypeError("timeout must be a float")
        for arg in ('cost', 'revision'):
            if (   locals()[arg] is not None
               and not isinstance(locals()[arg], (int, long))):
                raise TypeError("%s must be an integer" % arg)
        self.objid = objid
        self.repoid = repoid
        self.arch = arch
        self.basearch = basearch
        self.base_urls = list(base_urls)
        self.config_file = config_file
        self.cost = cost
        self.enabled = bool(enabled)
        self.gpg_check = bool(gpg_check)
        self.last_edit = last_edit
        self.last_update = last_update
        self.metalink = "" if not metalink else metalink
        self.mirror_list = "" if not mirror_list else mirror_list
        self.mirror_urls = [] if not mirror_urls else list(mirror_urls)
        self.name = name
        #self.pkg_count = pkg_count
        self.pkg_dir = pkg_dir
        self.ready = bool(ready)
        self.releasever = releasever
        self.repo_gpg_check = bool(repo_gpg_check)
        self.revision = revision
        self.timeout = timeout

    def __str__(self):
        return self.repoid

    def __getstate__(self):
        """
        Used for serialization with pickle.
        @return container content that will be serialized
        """
        return dict((k, getattr(self, k)) for k in self.__slots__)

    def __setstate__(self, state):
        """
        Used for deserialization with pickle.
        Restores the object from serialized form.
        @param state is an object created by __setstate__() method
        """
        for k, value in state.items():
            setattr(self, k, value)

def make_repository_from_db(repo):
    """
    Create instance of Repository from instance of yum.yumRepo.YumRepository
    @return instance of Repository
    """
    if not isinstance(repo, yum.yumRepo.YumRepository):
        raise TypeError("repo must be in instance of yum.yumRepo.YumRepository")
    metadata = {}
    for prop_name in Repository.__slots__[1:]:
        try:
            metadata[prop_name] = get_prop_from_yum_repo(repo, prop_name)
        except yum.Errors.RepoError as exc:
            # some properties can cause error (like requesting ready)
            logging.getLogger(__name__).warn(
                    'failed to get property "%s" of repo "%s": %s' % (
                        prop_name, repo.name, exc))
            if prop_name == "ready":
                metadata[prop_name] = False
            else:
                metadata[prop_name] = None

    res = Repository(id(repo), **metadata)
    return res




# -*- encoding: utf-8 -*-
# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Michal Minar <miminar@redhat.com>
#

"""
Just a common functionality related to SoftwareIdentityFileCheck provider.
"""

import hashlib
import os
import pywbem
import stat
import yum

from lmi.providers import cmpi_logging
from lmi.software import util
from lmi.software.yumdb import errors
from lmi.software.yumdb import jobs
from lmi.software.yumdb import packagecheck
from lmi.software.yumdb import packageinfo
from lmi.software.yumdb import YumDB

LOG = cmpi_logging.get_logger(__name__)

class Values(object):
    class TargetOperatingSystem(object):
        Unknown = pywbem.Uint16(0)
        Other = pywbem.Uint16(1)
        MACOS = pywbem.Uint16(2)
        ATTUNIX = pywbem.Uint16(3)
        DGUX = pywbem.Uint16(4)
        DECNT = pywbem.Uint16(5)
        Tru64_UNIX = pywbem.Uint16(6)
        OpenVMS = pywbem.Uint16(7)
        HPUX = pywbem.Uint16(8)
        AIX = pywbem.Uint16(9)
        MVS = pywbem.Uint16(10)
        OS400 = pywbem.Uint16(11)
        OS_2 = pywbem.Uint16(12)
        JavaVM = pywbem.Uint16(13)
        MSDOS = pywbem.Uint16(14)
        WIN3x = pywbem.Uint16(15)
        WIN95 = pywbem.Uint16(16)
        WIN98 = pywbem.Uint16(17)
        WINNT = pywbem.Uint16(18)
        WINCE = pywbem.Uint16(19)
        NCR3000 = pywbem.Uint16(20)
        NetWare = pywbem.Uint16(21)
        OSF = pywbem.Uint16(22)
        DC_OS = pywbem.Uint16(23)
        Reliant_UNIX = pywbem.Uint16(24)
        SCO_UnixWare = pywbem.Uint16(25)
        SCO_OpenServer = pywbem.Uint16(26)
        Sequent = pywbem.Uint16(27)
        IRIX = pywbem.Uint16(28)
        Solaris = pywbem.Uint16(29)
        SunOS = pywbem.Uint16(30)
        U6000 = pywbem.Uint16(31)
        ASERIES = pywbem.Uint16(32)
        HP_NonStop_OS = pywbem.Uint16(33)
        HP_NonStop_OSS = pywbem.Uint16(34)
        BS2000 = pywbem.Uint16(35)
        LINUX = pywbem.Uint16(36)
        Lynx = pywbem.Uint16(37)
        XENIX = pywbem.Uint16(38)
        VM = pywbem.Uint16(39)
        Interactive_UNIX = pywbem.Uint16(40)
        BSDUNIX = pywbem.Uint16(41)
        FreeBSD = pywbem.Uint16(42)
        NetBSD = pywbem.Uint16(43)
        GNU_Hurd = pywbem.Uint16(44)
        OS9 = pywbem.Uint16(45)
        MACH_Kernel = pywbem.Uint16(46)
        Inferno = pywbem.Uint16(47)
        QNX = pywbem.Uint16(48)
        EPOC = pywbem.Uint16(49)
        IxWorks = pywbem.Uint16(50)
        VxWorks = pywbem.Uint16(51)
        MiNT = pywbem.Uint16(52)
        BeOS = pywbem.Uint16(53)
        HP_MPE = pywbem.Uint16(54)
        NextStep = pywbem.Uint16(55)
        PalmPilot = pywbem.Uint16(56)
        Rhapsody = pywbem.Uint16(57)
        Windows_2000 = pywbem.Uint16(58)
        Dedicated = pywbem.Uint16(59)
        OS_390 = pywbem.Uint16(60)
        VSE = pywbem.Uint16(61)
        TPF = pywbem.Uint16(62)
        Windows__R__Me = pywbem.Uint16(63)
        Caldera_Open_UNIX = pywbem.Uint16(64)
        OpenBSD = pywbem.Uint16(65)
        Not_Applicable = pywbem.Uint16(66)
        Windows_XP = pywbem.Uint16(67)
        z_OS = pywbem.Uint16(68)
        Microsoft_Windows_Server_2003 = pywbem.Uint16(69)
        Microsoft_Windows_Server_2003_64_Bit = pywbem.Uint16(70)
        Windows_XP_64_Bit = pywbem.Uint16(71)
        Windows_XP_Embedded = pywbem.Uint16(72)
        Windows_Vista = pywbem.Uint16(73)
        Windows_Vista_64_Bit = pywbem.Uint16(74)
        Windows_Embedded_for_Point_of_Service = pywbem.Uint16(75)
        Microsoft_Windows_Server_2008 = pywbem.Uint16(76)
        Microsoft_Windows_Server_2008_64_Bit = pywbem.Uint16(77)
        FreeBSD_64_Bit = pywbem.Uint16(78)
        RedHat_Enterprise_Linux = pywbem.Uint16(79)
        RedHat_Enterprise_Linux_64_Bit = pywbem.Uint16(80)
        Solaris_64_Bit = pywbem.Uint16(81)
        SUSE = pywbem.Uint16(82)
        SUSE_64_Bit = pywbem.Uint16(83)
        SLES = pywbem.Uint16(84)
        SLES_64_Bit = pywbem.Uint16(85)
        Novell_OES = pywbem.Uint16(86)
        Novell_Linux_Desktop = pywbem.Uint16(87)
        Sun_Java_Desktop_System = pywbem.Uint16(88)
        Mandriva = pywbem.Uint16(89)
        Mandriva_64_Bit = pywbem.Uint16(90)
        TurboLinux = pywbem.Uint16(91)
        TurboLinux_64_Bit = pywbem.Uint16(92)
        Ubuntu = pywbem.Uint16(93)
        Ubuntu_64_Bit = pywbem.Uint16(94)
        Debian = pywbem.Uint16(95)
        Debian_64_Bit = pywbem.Uint16(96)
        Linux_2_4_x = pywbem.Uint16(97)
        Linux_2_4_x_64_Bit = pywbem.Uint16(98)
        Linux_2_6_x = pywbem.Uint16(99)
        Linux_2_6_x_64_Bit = pywbem.Uint16(100)
        Linux_64_Bit = pywbem.Uint16(101)
        Other_64_Bit = pywbem.Uint16(102)
        Microsoft_Windows_Server_2008_R2 = pywbem.Uint16(103)
        VMware_ESXi = pywbem.Uint16(104)
        Microsoft_Windows_7 = pywbem.Uint16(105)
        CentOS_32_bit = pywbem.Uint16(106)
        CentOS_64_bit = pywbem.Uint16(107)
        Oracle_Enterprise_Linux_32_bit = pywbem.Uint16(108)
        Oracle_Enterprise_Linux_64_bit = pywbem.Uint16(109)
        eComStation_32_bitx = pywbem.Uint16(110)
        Microsoft_Windows_Server_2011 = pywbem.Uint16(111)
        Microsoft_Windows_Server_2012 = pywbem.Uint16(113)
        Microsoft_Windows_8 = pywbem.Uint16(114)
        Microsoft_Windows_8_64_bit = pywbem.Uint16(115)
        _reverse_map = {
                0   : 'Unknown',
                1   : 'Other',
                2   : 'MACOS',
                3   : 'ATTUNIX',
                4   : 'DGUX',
                5   : 'DECNT',
                6   : 'Tru64 UNIX',
                7   : 'OpenVMS',
                8   : 'HPUX',
                9   : 'AIX',
                10  : 'MVS',
                11  : 'OS400',
                12  : 'OS/2',
                13  : 'JavaVM',
                14  : 'MSDOS',
                15  : 'WIN3x',
                16  : 'WIN95',
                17  : 'WIN98',
                18  : 'WINNT',
                19  : 'WINCE',
                20  : 'NCR3000',
                21  : 'NetWare',
                22  : 'OSF',
                23  : 'DC/OS',
                24  : 'Reliant UNIX',
                25  : 'SCO UnixWare',
                26  : 'SCO OpenServer',
                27  : 'Sequent',
                28  : 'IRIX',
                29  : 'Solaris',
                30  : 'SunOS',
                31  : 'U6000',
                32  : 'ASERIES',
                33  : 'HP NonStop OS',
                34  : 'HP NonStop OSS',
                35  : 'BS2000',
                36  : 'LINUX',
                37  : 'Lynx',
                38  : 'XENIX',
                39  : 'VM',
                40  : 'Interactive UNIX',
                41  : 'BSDUNIX',
                42  : 'FreeBSD',
                43  : 'NetBSD',
                44  : 'GNU Hurd',
                45  : 'OS9',
                46  : 'MACH Kernel',
                47  : 'Inferno',
                48  : 'QNX',
                49  : 'EPOC',
                50  : 'IxWorks',
                51  : 'VxWorks',
                52  : 'MiNT',
                53  : 'BeOS',
                54  : 'HP MPE',
                55  : 'NextStep',
                56  : 'PalmPilot',
                57  : 'Rhapsody',
                58  : 'Windows 2000',
                59  : 'Dedicated',
                60  : 'OS/390',
                61  : 'VSE',
                62  : 'TPF',
                63  : 'Windows (R) Me',
                64  : 'Caldera Open UNIX',
                65  : 'OpenBSD',
                66  : 'Not Applicable',
                67  : 'Windows XP',
                68  : 'z/OS',
                69  : 'Microsoft Windows Server 2003',
                70  : 'Microsoft Windows Server 2003 64-Bit',
                71  : 'Windows XP 64-Bit',
                72  : 'Windows XP Embedded',
                73  : 'Windows Vista',
                74  : 'Windows Vista 64-Bit',
                75  : 'Windows Embedded for Point of Service',
                76  : 'Microsoft Windows Server 2008',
                77  : 'Microsoft Windows Server 2008 64-Bit',
                78  : 'FreeBSD 64-Bit',
                79  : 'RedHat Enterprise Linux',
                80  : 'RedHat Enterprise Linux 64-Bit',
                81  : 'Solaris 64-Bit',
                82  : 'SUSE',
                83  : 'SUSE 64-Bit',
                84  : 'SLES',
                85  : 'SLES 64-Bit',
                86  : 'Novell OES',
                87  : 'Novell Linux Desktop',
                88  : 'Sun Java Desktop System',
                89  : 'Mandriva',
                90  : 'Mandriva 64-Bit',
                91  : 'TurboLinux',
                92  : 'TurboLinux 64-Bit',
                93  : 'Ubuntu',
                94  : 'Ubuntu 64-Bit',
                95  : 'Debian',
                96  : 'Debian 64-Bit',
                97  : 'Linux 2.4.x',
                98  : 'Linux 2.4.x 64-Bit',
                99  : 'Linux 2.6.x',
                100 : 'Linux 2.6.x 64-Bit',
                101 : 'Linux 64-Bit',
                102 : 'Other 64-Bit',
                103 : 'Microsoft Windows Server 2008 R2',
                104 : 'VMware ESXi',
                105 : 'Microsoft Windows 7',
                106 : 'CentOS 32-bit',
                107 : 'CentOS 64-bit',
                108 : 'Oracle Enterprise Linux 32-bit',
                109 : 'Oracle Enterprise Linux 64-bit',
                110 : 'eComStation 32-bitx',
                111 : 'Microsoft Windows Server 2011',
                113 : 'Microsoft Windows Server 2012',
                114 : 'Microsoft Windows 8',
                115 : 'Microsoft Windows 8 64-bit'
        }

    class ChecksumType(object):
        UNKNOWN = pywbem.Uint16(0)
        MD5 = pywbem.Uint16(1)
        SHA_1 = pywbem.Uint16(2)
        RIPE_MD_160 = pywbem.Uint16(3)
        SHA256 = pywbem.Uint16(8)
        SHA384 = pywbem.Uint16(9)
        SHA512 = pywbem.Uint16(10)
        SHA224 = pywbem.Uint16(11)
        _reverse_map = {
                0  : 'UNKNOWN',
                1  : 'MD5',
                2  : 'SHA-1',
                3  : 'RIPE-MD/160',
                8  : 'SHA256',
                9  : 'SHA384',
                10 : 'SHA512',
                11 : 'SHA224'
        }

    class FileType(object):
        Unknown = pywbem.Uint16(0)
        File = pywbem.Uint16(1)
        Directory = pywbem.Uint16(2)
        Symlink = pywbem.Uint16(3)
        FIFO = pywbem.Uint16(4)
        Character_Device = pywbem.Uint16(5)
        Block_Device = pywbem.Uint16(6)
        _reverse_map = {
                0 : 'Unknown',
                1 : 'File',
                2 : 'Directory',
                3 : 'Symlink',
                4 : 'FIFO',
                5 : 'Character Device',
                6 : 'Block Device'
        }

    class FileModeFlags(object):
        Execute_Other = pywbem.Uint8(0)
        Write_Other = pywbem.Uint8(1)
        Read_Other = pywbem.Uint8(2)
        Execute_Group = pywbem.Uint8(3)
        Write_Group = pywbem.Uint8(4)
        Read_Group = pywbem.Uint8(5)
        Execute_User = pywbem.Uint8(6)
        Write_User = pywbem.Uint8(7)
        Read_User = pywbem.Uint8(8)
        Sticky = pywbem.Uint8(9)
        SGID = pywbem.Uint8(10)
        SUID = pywbem.Uint8(11)
        _reverse_map = {
                0  : 'Execute Other',
                1  : 'Write Other',
                2  : 'Read Other',
                3  : 'Execute Group',
                4  : 'Write Group',
                5  : 'Read Group',
                6  : 'Execute User',
                7  : 'Write User',
                8  : 'Read User',
                9  : 'Sticky',
                10 : 'SGID',
                11 : 'SUID'
        }

    class FileModeFlagsOriginal(object):
        Execute_Other = pywbem.Uint8(0)
        Write_Other = pywbem.Uint8(1)
        Read_Other = pywbem.Uint8(2)
        Execute_Group = pywbem.Uint8(3)
        Write_Group = pywbem.Uint8(4)
        Read_Group = pywbem.Uint8(5)
        Execute_User = pywbem.Uint8(6)
        Write_User = pywbem.Uint8(7)
        Read_User = pywbem.Uint8(8)
        Sticky = pywbem.Uint8(9)
        SGID = pywbem.Uint8(10)
        SUID = pywbem.Uint8(11)
        _reverse_map = {
                0  : 'Execute Other',
                1  : 'Write Other',
                2  : 'Read Other',
                3  : 'Execute Group',
                4  : 'Write Group',
                5  : 'Read Group',
                6  : 'Execute User',
                7  : 'Write User',
                8  : 'Read User',
                9  : 'Sticky',
                10 : 'SGID',
                11 : 'SUID'
        }

    class FailedFlags(object):
        Existence = pywbem.Uint16(0)
        FileSize = pywbem.Uint16(1)
        FileMode = pywbem.Uint16(2)
        Checksum = pywbem.Uint16(3)
        Device_Number = pywbem.Uint16(4)
        LinkTarget = pywbem.Uint16(5)
        UserID = pywbem.Uint16(6)
        GroupID = pywbem.Uint16(7)
        Last_Modification_Time = pywbem.Uint16(8)
        _reverse_map = {
                0 : 'Existence',
                1 : 'FileSize',
                2 : 'FileMode',
                3 : 'Checksum',
                4 : 'Device Number',
                5 : 'LinkTarget',
                6 : 'UserID',
                7 : 'GroupID',
                8 : 'Last Modification Time'
        }

    class SoftwareElementState(object):
        Deployable = pywbem.Uint16(0)
        Installable = pywbem.Uint16(1)
        Executable = pywbem.Uint16(2)
        Running = pywbem.Uint16(3)
        _reverse_map = {
                0 : 'Deployable',
                1 : 'Installable',
                2 : 'Executable',
                3 : 'Running'
        }

    class FileTypeOriginal(object):
        Unknown = pywbem.Uint16(0)
        File = pywbem.Uint16(1)
        Directory = pywbem.Uint16(2)
        Symlink = pywbem.Uint16(3)
        FIFO = pywbem.Uint16(4)
        Character_Device = pywbem.Uint16(5)
        Block_Device = pywbem.Uint16(6)
        _reverse_map = {
                0 : 'Unknown',
                1 : 'File',
                2 : 'Directory',
                3 : 'Symlink',
                4 : 'FIFO',
                5 : 'Character Device',
                6 : 'Block Device'
        }

    class Invoke(object):
        """
        This is not an enumeration from mof file, it just gives human
        names to numeric values.
        """
        Satisfied = pywbem.Uint32(0)
        Not_Supported = pywbem.Uint32(1)
        Not_Satisfied = pywbem.Uint32(2)

class FileCheck(object):
    """
    File attribute storage keeping both RPM database metadata and info
    from local filesystem for single file. Most attributes are loaded at the
    time of first request and cached for later use.

    Most of properties return tuple:
        ``(installed, original)``
    where
        ``installed`` is a state of property of installed file, while
        ``original`` is the value stored in RPM database.

    If any of those values are None, it means that property could not be
    retrieved or it's not applicable to file's type.
    """

    def __init__(self, pkg_info, pkg_file, checksum_type):
        """
        :param pkg_file: (``yumdb.packagecheck.PackageFile``) Metadata for
            file loaded from rpm database.
        :param checksum_type: (``int``) Indentificator of checksum algorithm
            corresponding to values in yum.constants.RPM_CHECKSUM_TYPES.
        """
        if not isinstance(pkg_info, packageinfo.PackageInfo):
            raise TypeError("pkg_info must be an instance of PackageInfo")
        if not isinstance(pkg_file, packagecheck.PackageFile):
            raise TypeError("pkg_file must be an instance of PackageFile")
        self._pkg_info = pkg_info
        self._pkg_file = pkg_file
        self._exists = None
        # (md5 checksum, rpm-type checksum) - this is made on first request
        self._checksums = None
        self._checksum_type = checksum_type
        self._link_target = None
        # stores result of os.stat() call
        self._stat = None

    @property
    def pkg_info(self):
        """:rtype: (``PackageInfo``)"""
        return self._pkg_info

    @property
    def pkg_file(self):
        """:rtype: (``PackageFile``)"""
        return self._pkg_file

    @property
    def path(self):
        """Return absolute file path."""
        return self._pkg_file.path

    @property
    def exists(self):
        """Return true if file exists on local file system."""
        if self._exists is None:
            self._exists = os.path.lexists(self._pkg_file.path)
        return self._exists

    @property
    def stat(self):
        """
        Return cached stat object - result of ``os.lstat()``.
        None is returned if file does not exist.
        """
        if self._stat is None and self.exists:
            self._stat = os.lstat(self.path)
        return self._stat

    @property
    def file_type(self):
        """
        Return identifiers of file type for installed and rpm file.
        If the installed file does not exist, None is returned on
        its position.
        :rtype: (``tuple``) Pair of (installed, original).
        """
        fm = self.file_mode[0]
        if fm is None:
            ft = None
        elif stat.S_ISLNK(fm):
            ft = packagecheck.FILE_TYPE_SYMLINK
        elif stat.S_ISDIR(fm):
            ft = packagecheck.FILE_TYPE_DIRECTORY
        elif stat.S_ISFIFO(fm):
            ft = packagecheck.FILE_TYPE_FIFO
        elif stat.S_ISCHR(fm):
            ft = packagecheck.FILE_TYPE_CHARACTER_DEVICE
        elif stat.S_ISBLK(fm):
            ft = packagecheck.FILE_TYPE_BLOCK_DEVICE
        elif stat.S_ISREG(fm):
            ft = packagecheck.FILE_TYPE_FILE
        else:
            ft = packagecheck.FILE_TYPE_UNKNOWN
        return (ft, self._pkg_file.file_type)

    @property
    def md5_checksum(self):
        """
        Return md5 checksum string of installed file.
        This is computed on first access either to this or ``file_checksum``
        property and cached for later access.
        :rtype: (``str``)
        """
        if self._checksums is None:
            self.make_checksums()
        return self._checksums[0] if self._checksums else None

    @property
    def file_checksum_type(self):
        """
        Return identifier of hash algorithm used in rpm package.
        :rtype: (``int``)
        """
        return self._checksum_type

    @property
    def file_checksum(self):
        """
        Return checksum strings for installed and rpm file.
        This is computed on first access either to this or ``md5_checksum``
        property and cached for later access.
        :rtype: (``tuple``) Pair of (installed, original).
        """
        if self._checksums is None:
            self.make_checksums()
        return ( self._checksums[1] if self._checksums else None
               , self._pkg_file.checksum)

    @property
    def file_size(self):
        """:rtype: (``tuple``) Pair of (installed, original)."""
        return (self.getstat('size'), self._pkg_file.size)

    @property
    def file_mode(self):
        """:rtype: (``tuple``) Pair of (installed, original)."""
        return (self.getstat('mode'), self._pkg_file.mode)

    @property
    def device(self):
        """
        :returns: Pair of (installed, original). Where both are raw device
            numbers. ``os.major()`` and ``os.minor()`` function can be used
            on them.
        :rtype: tuple
        """
        return (self.getstat('rdev'), self._pkg_file.device)

    @property
    def link_target(self):
        """:rtype: (``tuple``) Pair of (installed, original)."""
        if (   self.file_type[0] == packagecheck.FILE_TYPE_SYMLINK
           and self._link_target is None):
            self._link_target = os.readlink(self.path)
        return (self._link_target, self._pkg_file.link_target)

    @property
    def user_id(self):
        """:rtype: (``tuple``) Pair of (installed, original)."""
        return (self.getstat('uid'), self._pkg_file.uid)

    @property
    def group_id(self):
        """:rtype: (``tuple``) Pair of (installed, original)."""
        return (self.getstat('gid'), self._pkg_file.gid)

    @property
    def last_modification_time(self):
        """:rtype: (``tuple``) Pair of (installed, original)."""
        return (self.getstat('mtime'), self._pkg_file.mtime)

    def getstat(self, attr):
        """
        Get attribute from stat object for given file.
        :param attr: (``str``) Will be prefixed with "st_".
        """
        if self.stat is not None:
            return getattr(self.stat, "st_"+attr)

    def make_checksums(self):
        """
        Compute checksums for installed file and cache them.
        :rtype: (``tuple``) MD5 and rpm-type checksums for given file
            as a pair.
        """
        if self.file_type[0] == packagecheck.FILE_TYPE_FILE:
            self._checksums = compute_checksums(
                    self._checksum_type, self.path)
        else:
            self._checksums = None
        return self._checksums


@cmpi_logging.trace_function
def hashfile(afile, hashers, blocksize=65536):
    """
    Generic function computing multiple hashes from single file at once.

    :param hashers: (``list``) A list of hash objects.
    :rtype: (``list``) List of digest strings (in hex format) for each
        given hash object in the same order.
    """
    if not isinstance(hashers, (tuple, list, set, frozenset)):
        hashers = (hashers, )
    buf = afile.read(blocksize)
    while len(buf) > 0:
        for hashfunc in hashers:
            hashfunc.update(buf)
        buf = afile.read(blocksize)
    return [ hashfunc.hexdigest() for hashfunc in hashers ]

@cmpi_logging.trace_function
def compute_checksums(checksum_type, file_path):
    """
    Compute file checksums in one go.

    :param checksum_type: (``int`) Selected hash algorithm to compute second
        checksum (the one used for rpm package). Correct value can be
        obtained from ``yum.constants.RPM_CHECKSUM_TYPES``.
    :param file_path: (``str``) Absolute path of regular file to hash.
    :rtype (md5sum, checksum) Tuple of strings containing hex represention
        of computed checksums.

    Both checksums are computed from file_path's content.
    First one is always md5, the second one depends on checksum_type.
    If file does not exists, (None, None) is returned.
    """
    LOG().debug("checksuming file %s", file_path)
    hashers = [hashlib.md5()]   #pylint: disable=E1101
    if checksum_type != packagecheck.CHECKSUMTYPE_STR2NUM["md5"]:
        hashers.append(checksumtype_num2hash(checksum_type)())
    try:
        with open(file_path, 'rb') as fobj:
            rslts = hashfile(fobj, hashers)
    except (OSError, IOError) as exc:
        LOG().error("could not open file \"%s\" for reading: %s",
                file_path, exc)
        return None, None
    return (rslts[0], rslts[1] if len(rslts) > 1 else rslts[0])

@cmpi_logging.trace_function
def checksumtype_num2hash(csumt):
    """
    Get checksum function for rpm hash identifier.

    :param csumt: (``int``) Checksum type as a number obtained from package.
    :rtype: (``function``) Hash function object corresponding to csumt.
    """
    return getattr(hashlib, yum.constants.RPM_CHECKSUM_TYPES[csumt])

@cmpi_logging.trace_function
def mode2pywbem_flags(mode):
    """
    Utility for creation of value of ``FileModeFlags`` property out of file's
    raw mode.

    :param mode: (``int``) File's mode. Retrieved from os.lstat().
        If None, file does not exist.
    :rtype: (``list``) Of integer flags describing file's access permissions.
    """
    if mode is None:
        return None
    flags = []
    for i, flag in enumerate((
            stat.S_IXOTH,
            stat.S_IWOTH,
            stat.S_IROTH,
            stat.S_IXGRP,
            stat.S_IWGRP,
            stat.S_IRGRP,
            stat.S_IXUSR,
            stat.S_IWUSR,
            stat.S_IRUSR,
            stat.S_ISVTX,
            stat.S_ISGID,
            stat.S_ISUID)):
        if flag & mode:
            flags.append(pywbem.Uint8(i))
    return flags

@cmpi_logging.trace_function
def get_existing_check_from_job(check_id):
    """
    ``CheckID`` of ``LMI_SoftwareIdentityFileCheck`` refers to asynchronous job
    that caused creation of this file check. Let's parse it, get the job out
    of YumWorker and ask it for results so they don't have to be computed
    again.

    :param check_id: (``str``) Value of ``CheckID`` property.
    :rtype: (``tuple``) Pair of (pkg_info, pkg_check).
        ``None`` if job does not exists anymore.
    """
    try:
        match = util.RE_INSTANCE_ID.match(check_id)
        if not match:
            raise ValueError(check_id)
        if (  not match.group('clsname').lower()
           == "LMI_SoftwareVerificationJob".lower()):
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "CheckID does not contain supported job class name: "
                    " %s != %s" % (match.group('clsname'),
                        "LMI_SoftwareVerificationJob".lower()))
        job_id = int(match.group('id'))
        job = YumDB.get_instance().get_job(job_id)
        if not isinstance(job,
                (jobs.YumCheckPackage, jobs.YumCheckPackageFile)):
            LOG().error('CheckID="%s" of LMI_SoftwareIdentityFileCheck refers'
                ' to job %s.', check_id, job)
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                "CheckID refers to wrong job.")
        return job.result_data

    except errors.JobNotFound:
        # allow this - job could already be deleted
        LOG().warn("could not find YumCheckPackage or"
                " YumCheckPackageFile job with id \"%d\"" % job_id)
    except ValueError:
        raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                "Could not parse \"CheckID\" key property: \"%s\"." % check_id)

@cmpi_logging.trace_function
def object_path2file_check(objpath):
    """
    @return (package_info, package_check, file_name)
    """
    if not isinstance(objpath, pywbem.CIMInstanceName):
        raise TypeError("objpath must be instance of CIMInstanceName, "
                "not \"%s\"" % objpath.__class__.__name__)

    for key in ('Name', 'SoftwareElementID', 'Version',
            'SoftwareElementState', 'TargetOperatingSystem'):
        if not key in objpath or not objpath[key]:
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                    "Missing key property \"%s\"." % key)
    version_match = util.RE_EVRA.match(objpath['Version'])
    if not version_match:
        raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
            'Failed to parse \"Version\" property for valid EVRA: "%s".' %
            objpath['Version'])
    pkg_fltr = util.nevra2filter(objpath["SoftwareElementID"])
    if any(   pkg_fltr[attr] != version_match.group(attr)
          for attr in ('epoch', 'version', 'release', 'arch')):
        raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
            'Inconsistent \"Version\" and \"SoftwareElementID\" key'
            ' properties.')
    if (  objpath['SoftwareElementState']
       != Values.SoftwareElementState.Executable):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                "Only \"Executable\" software element state supported")
    if not util.check_target_operating_system(
            objpath['TargetOperatingSystem']):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                "Wrong target operating system.")

    ydb = YumDB.get_instance()
    pkg_check = None
    if (  objpath["CheckID"].lower()
       != "LMI:LMI_SoftwareIdentityFileCheck".lower()):
        check = get_existing_check_from_job(objpath["CheckID"])
        if check is not None:
            pkg_info, pkg_check = check
        # else - job does not exist (could be deleted)

    if pkg_check is None:
        # let the YumWorker check the rest
        try:
            pkg_info, pkg_check = ydb.check_package_file(
                    objpath["SoftwareElementID"], objpath["Name"])
        except errors.PackageNotFound:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                "Could not find package matching SoftwareElementID \"%s\"" %
                objpath["SoftwareElementID"])
        except errors.FileNotFound as exc:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND, exc.message)

    # last check for file path
    if objpath["Name"] not in pkg_check:
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
            'File \"%s\" not found in package \"%s\".' %
            objpath["SoftwareElementID"])
    return FileCheck(pkg_info, pkg_check[objpath["Name"]],
            pkg_check.file_checksum_type)

@cmpi_logging.trace_function
def test_file(pkg_info, checksum_type, pkg_file):
    """
    :param checksum_type: (``int``) Identifier of checksum algorithm used
        in rpm package. It's one of keys of
        ``yum.constants.RPM_CHECKSUM_TYPES`` dictionary.
    :rtype: (``FileCheck``)
    """
    for var, cls in (
                ("pkg_info", packageinfo.PackageInfo),
                ("pkg_file", packagecheck.PackageFile)):
        if not isinstance(locals()[var], cls):
            raise TypeError("%s must be an instance of %s, not %s" % (
                var, cls.__name__, pkg_file.__class__.__name__))
    return FileCheck(pkg_info, pkg_file, checksum_type)

@cmpi_logging.trace_function
def _file_check2failed_flags(file_check):
    """
    Makes value of ``FailedFlags`` property from ``FileCheck`` instance.

    :param file_check: (``FileCheck``)
    :rtype: (``pywbem.CIMProperty``) Property ``FailedFlags`` value.
    """
    if not isinstance(file_check, FileCheck):
        raise TypeError("file_check must be an instance of FileCheck")

    if not file_check.exists:
        return [Values.FailedFlags.Existence]
    flags = set()
    fails_on = lambda attr: (
            getattr(file_check, attr)[0] != getattr(file_check, attr)[1])
    if fails_on('file_type'):
        flags.add(Values.FailedFlags.FileMode)

    # file type specific checks
    if file_check.file_type[1] == packagecheck.FILE_TYPE_FILE:
        if fails_on('file_size'):
            flags.add(Values.FailedFlags.FileSize)
        if fails_on('last_modification_time'):
            flags.add(Values.FailedFlags.Last_Modification_Time)
        if fails_on('file_checksum'):
            flags.add(Values.FailedFlags.Checksum)
    elif file_check.file_type[1] in (
            packagecheck.FILE_TYPE_CHARACTER_DEVICE,
            packagecheck.FILE_TYPE_BLOCK_DEVICE):
        if fails_on('device'):
            flags.add(Values.FailedFlags.Device_Number)
    elif file_check.file_type[1] == packagecheck.FILE_TYPE_SYMLINK:
        if fails_on('link_target'):
            flags.add(Values.FailedFlags.LinkTarget)

    # generic checks
    if (   fails_on('file_mode')
       # do not check permissions for symlinks
       and file_check.file_type[1] != packagecheck.FILE_TYPE_SYMLINK):
        flags.add(Values.FailedFlags.FileMode)
    if fails_on('user_id'):
        flags.add(Values.FailedFlags.UserID)
    if fails_on('group_id'):
        flags.add(Values.FailedFlags.GroupID)

    return list(flags)

@cmpi_logging.trace_function
def file_check_passed(file_check):
    """
    Verify single file and return true if it passes.
    """
    return len(_file_check2failed_flags(file_check)) == 0

@cmpi_logging.trace_function
def _fill_non_key_values(model, file_check):
    """
    Fills a non key values into instance of SoftwareIdentityFileCheck.

    :param model: (``CIMInstance``) Model, that will be filled with
        all supported non-key properties.
    """
    set_prop = lambda name, kwargs: \
            model.__setitem__(name, pywbem.CIMProperty(name, **kwargs))

    set_prop("CheckMode", dict(type='boolean', value=True))
    model["ChecksumType"] = pywbem.Uint16(file_check.file_checksum_type)

    set_prop("FailedFlags", dict(type='uint16', is_array=True,
            value=_file_check2failed_flags(file_check)))
    for mattr, fattr, kwargs, convert in (
            ('LastModificationTime', 'last_modification_time',
                {'type':'uint64'}, pywbem.Uint64),
            ('FileType'    , 'file_type'    ,{'type':'uint16'}, pywbem.Uint16),
            ('UserID'      , 'user_id'      ,{'type':'uint32'}, pywbem.Uint32),
            ('GroupID'     , 'group_id'     ,{'type':'uint32'}, pywbem.Uint32),
            ('FileMode'    , 'file_mode'    ,{'type':'uint32'}, pywbem.Uint32),
            ('FileSize'    , 'file_size'    ,{'type':'uint64'}, pywbem.Uint64),
            ('LinkTarget'  , 'link_target'  ,{'type':'string'}, str),
            ('FileChecksum', 'file_checksum',{'type':'string'}, str)):
        installed, orig = getattr(file_check, fattr)
        kwargs['value'] = None if orig is None else convert(orig)
        set_prop(mattr+"Original", kwargs)
        kwargs['value'] = None if installed is None else convert(installed)
        set_prop(mattr, kwargs)
    model['FileName'] = os.path.basename(file_check.path)
    for suf in ('', 'Original'):
        set_prop('FileModeFlags'+suf, dict(type='uint8', is_array=True,
            value=mode2pywbem_flags(file_check.file_mode[1])))
    set_prop("FileExists", dict(type='boolean', value=file_check.exists))
    set_prop('MD5Checksum', dict(type='string',
            value=file_check.md5_checksum))

@cmpi_logging.trace_function
def file_check2model(file_check, keys_only=True, model=None, job=None):
    """
    :param file_check: (``FileCheck``) File check information to transform
        to cim instance.
    :param keys_only: (``bool``) Whether to fill non-key properties.
    :param model: (``CIMInstance`` | ``CIMInstanceName``) If given,
        it will be used as a template. Only property values will be set.
        This instance will be returned on completion.
    :param job: (``YumAsyncJob``) If this is a consequence of asynchronous
        job operation, the instance of job should be supplied so that
        ``CheckID`` property can be filled.
    :rtype: (``CIMInstance`` | ``CIMInstanceName``) Object of
        LMI_SoftwareIdentityFileCheck with filled desired values.
    """
    if job is not None and not isinstance(job, jobs.YumCheckPackage):
        raise TypeError("job must be instance of jobs.YumCheckPackage")
    if not isinstance(file_check, FileCheck):
        raise TypeError("file_check must be an instance of FileCheck")

    if model is None:
        model = util.new_instance_name("LMI_SoftwareIdentityFileCheck")
        if not keys_only:
            model = pywbem.CIMInstance("LMI_SoftwareIdentityFileCheck",
                    path=model)

    if not keys_only:
        model.path.update(  #pylint: disable=E1103
            {k: None for k in ("Name", "SoftwareElementID",
            "SoftwareElementState", "TargetOperatingSystem", "Version",
            "CheckID")})

    model['Name'] = file_check.path
    model['SoftwareElementID'] = file_check.pkg_info.nevra
    model['SoftwareElementState'] = Values.SoftwareElementState.Executable
    model['TargetOperatingSystem'] = pywbem.Uint16(
            util.get_target_operating_system()[0])
    model['Version'] = file_check.pkg_info.evra
    model['CheckID'] = ("LMI:LMI_SoftwareVerificationJob:%d" % job.jobid
        if job is not None else "LMI:%s" % model.classname)
    if not keys_only:
        _fill_non_key_values(model, file_check)
    return model




# -*- encoding: utf-8 -*-
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Roman Rakus <rrakus@redhat.com>
#

import subprocess

def user_exists(username):
    """
    Return true/false if user does/does not exists
    """
    got = field_in_passwd(username, 0)
    return got == username

def group_exists(groupname):
    """
    Return true/false if user does/does not exists
    """
    got = field_in_group(groupname, 0)
    return got == groupname

def field_in_passwd(username, number):
    """
    Return numberth field in /etc/passwd for given username
    """
    for line in open("/etc/passwd").readlines():
        if line.startswith(username):
            return line.split(":")[number].strip()

def field_in_shadow(username, number):
    """
    Return numberth field in /etc/shadow for given username
    """
    for line in open("/etc/shadow").readlines():
        if line.startswith(username):
            return line.split(":")[number].strip()

def field_in_group(groupname, number):
    """
    Return numberth field in /etc/group for given groupname
    """
    for line in open("/etc/group").readlines():
        if line.startswith(groupname):
            return line.split(":")[number].strip()

def clean_account(user_name):
    """
    Force to delete testing account and remove home dir
    """
    if user_exists(user_name):
        subprocess.check_call(["userdel", "-fr", user_name])
    if group_exists(user_name):
        # groups should be expicitely deleted
        subprocess.check_call(["groupdel", user_name])

def add_user_to_group(user_name, group_name):
    """
    Will add user to group
    """
    subprocess.check_call(["usermod", "-a", "-G", group_name, user_name])

def create_account(user_name):
    """
    Force to create account; run clean_account before creation
    """
    if not user_exists(user_name):
        subprocess.check_call(["useradd", user_name])

def clean_group(group_name):
    """
    Force to delete testing group
    """
    if group_exists(group_name):
        subprocess.check_call(["groupdel", group_name])

def create_group(group_name):
    """
    Force to create group
    """
    if not group_exists(group_name):
        subprocess.check_call(["groupadd", group_name])




#!/usr/bin/env python
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Michal Minar <miminar@redhat.com>
#
"""
Module defining and creating testing packages and repositories for
OpenLMI Software provider.
"""

import collections
import datetime
import functools
import json
import os
import shutil
import stat
import subprocess
import tempfile
import time
import yum

import package
import repository
import util

REGULAR, DOC, CONFIG, LINK, CHARDEV, BLOCKDEV, FIFO = [
        2**i for i in range(7) ]

#: Holds variables for yum config.
#: * ``releasever`` - version of distribution ('18' for *Fedora 18*).
#: * ``disttag`` - dist tag of distribution. This is a part of release string
#:                 of packages build for particular distribution
#:                 ('.fc18' for *Fedora 18*).
#: * ``basearch`` - target architecture ('i686', 'x86_64', etc.).
#: * ``arch`` - system architecture ('ia32e')
YumVars = collections.namedtuple('YumVars',
        ('releasever', 'disttag', 'basearch', 'arch'))

#: Dictionary with shortned package names as keys with assigned contents.
#: Content represented with recursive dictionaries where dictionaries
#: represent folders and tuples files, pipes, symlinks, etc.
#: Directories whose names end with '/' will be owned.
PKG_FILE_ENTRIES = {
    'pkg1' : {
        'usr/share/openlmi-sw-test-pkg1/' : {
            'README'    : (REGULAR | DOC, 0644,  'This is a test pkg 1\n'),
            'Changelog' : (REGULAR | DOC, 0644, 'Nothing new\n'),
            'data/' : { 'file.txt' : (REGULAR, 0644, 'content\n') }
        },
        'etc/openlmi/software/test/' : {
            'dummy_config.cfg' :
                (REGULAR | CONFIG, 0644, '# This is a dummy cfg file\n'),
        }
    },

    'pkg2' : {
        'usr/share/openlmi-sw-test-pkg2/' : {
            'README'    : (REGULAR | DOC, 0644, 'This is a test pkg 2\n'),
            'Changelog' : (REGULAR | DOC, 0644, 'Nothing new\n'),
            'data/' : {
                'target.txt' : (REGULAR, 0644, 'Target of symlinks\n')
            },
            'symlinks/' : {
                'relative' : (LINK, 0644, '../data/target.txt'),
                'absolute' : (LINK, 0644,
                    'usr/share/openlmi-sw-text-pkg1/data/target.txt')
            }
        },
    },

    'pkg3' : {
        'usr/share/openlmi-sw-test-pkg3/' : {
            'README'    : (REGULAR | DOC, 0644, 'This is a test pkg 3\n'),
            'Changelog' : (REGULAR | DOC, 0644, 'Nothing new\n'),
            'fifos/'    : {
                'pipe' : (FIFO, 0644, None)
            },
            'devs/'     : {
                'char12'  : (CHARDEV, 0644, (1, 2)),
                'char34'  : (CHARDEV, 0644, (3, 4)),
                'block56' : (BLOCKDEV, 0644, (5, 6)),
                'block78' : (BLOCKDEV, 0644, (7, 8))
            }
        }
    },

    'pkg4' : {
        'usr/bin' : {
            'openlmi-sw-test-script' :
                (REGULAR, 0755, '#!/bin/sh\necho "OpenLMI rules!"')
        },
        'usr/share/openlmi-sw-test-pkg4/' : {
            'README'    : (REGULAR | DOC, 0644, 'This is a test pkg 4\n'),
            'Changelog' : (REGULAR | DOC, 0644, 'Nothing new\n'),
            'perms/' : {    # this will be given 777 perms with the sticky bit
                'rwxrwxrwx' : (REGULAR, 0777, ''),
                'rw-rw-rw-' : (REGULAR, 0666, ''),
                'r--r--r--' : (REGULAR, 0444, ''),
                'r-xr-xr-x' : (REGULAR, 0555, ''),
                'r---w---x' : (REGULAR, 0421, ''),
                'rwSr--r--' : (REGULAR, 04644, ''),
                'rw-r-Sr--' : (REGULAR, 02644, ''),
                'rw-r--r-T' : (REGULAR, 01644, ''),
                'rwsr-sr-t' : (REGULAR, 07755, ''),
            }
        },
    }
}

#: Repositories with assigned packages.
REPOS = {
    'openlmi-sw-test-repo-stable' : {
        # grows by revision
        'openlmi-sw-test-pkg1-0:1.2.3-1%(disttag)s.noarch' : {
            'files' : PKG_FILE_ENTRIES['pkg1']
        },
        # grows by epoch
        'openlmi-sw-test-pkg2-1:1.2.3-1%(disttag)s.noarch' : {
            'files' : PKG_FILE_ENTRIES['pkg2']
        },
        # grows by version
        'openlmi-sw-test-pkg3-0:1.2.1-3%(disttag)s.noarch' : {
            'files' : PKG_FILE_ENTRIES['pkg3']
        },
        # no update available
        'openlmi-sw-test-pkg4-1:1.2.3-1%(disttag)s.noarch' : {
            'files' : PKG_FILE_ENTRIES['pkg4']
        }
    },

    'openlmi-sw-test-repo-updates' : {
        'openlmi-sw-test-pkg1-0:1.2.3-2%(disttag)s.noarch' : {
            'files' : PKG_FILE_ENTRIES['pkg1']
        },
        'openlmi-sw-test-pkg2-2:1.2.2-1%(disttag)s.noarch' : {
            'files' : PKG_FILE_ENTRIES['pkg2']
        },
        'openlmi-sw-test-pkg3-0:1.2.2-2%(disttag)s.noarch' : {
            'files' : PKG_FILE_ENTRIES['pkg3']
        },
        # exactly the same version as in repo stable
        'openlmi-sw-test-pkg4-1:1.2.3-1%(disttag)s.noarch' : {
            'files' : PKG_FILE_ENTRIES['pkg4']
        }
    },

    'openlmi-sw-test-repo-updates-testing' : {
        'openlmi-sw-test-pkg1-0:1.2.3-3%(disttag)s.noarch' : {
            'files' : PKG_FILE_ENTRIES['pkg1']
        },
        'openlmi-sw-test-pkg2-3:1.2.0-1%(disttag)s.noarch' : {
            'files' : PKG_FILE_ENTRIES['pkg2']
        },
    },

    'openlmi-sw-test-repo-misc' : {
        # this package conflicts with pkg1 (explicitely)
        'openlmi-sw-test-conflict1-0:1.2.3-1%(disttag)s.noarch' : {
            'conflicts' : ['openlmi-sw-test-pkg1'],
            'files' : {
                'usr/share/openlmi-sw-test-conflict1/' : {
                    'README'    : (REGULAR | DOC, 0644,  ''),
                }
            }
        },
        # this package conflicts with pkg2 (by files)
        'openlmi-sw-test-conflict2-0:1.2.3-1%(disttag)s.noarch' : {
            'files' : PKG_FILE_ENTRIES['pkg1']
        },
        # this package depends on few packages above
        'openlmi-sw-test-depend1-0:1.2.3-1%(disttag)s.noarch' : {
            'requires' : ['openlmi-sw-test-pkg1', 'openlmi-sw-test-pkg2'],
            'files' : {
                'usr/share/openlmi-sw-test-depend1/' : {
                    'README'    : (REGULAR | DOC, 0644,  ''),
                }
            }
        },
        # same as previous one, with version requirements
        'openlmi-sw-test-depend2-0:1.2.3-1%(disttag)s.noarch' : {
            'requires' : [
                'openlmi-sw-test-pkg1 = 1.2.3-1%(disttag)s',
                'openlmi-sw-test-pkg2 = 2:1.2.2-1%(disttag)s'],
            'files' : {
                'usr/share/openlmi-sw-test-depend2/' : {
                    'README'    : (REGULAR | DOC, 0644,  ''),
                }
            }
        },
        # same package as stable#pkg2 except for arch and shared
        # directory
        'openlmi-sw-test-pkg2-1:1.2.3-1%(disttag)s.%(basearch)s' : {
            'files' : {
                'usr/share/openlmi-sw-test-pkg2-arch/' :
                    PKG_FILE_ENTRIES['pkg2'].values()[0]
            }
        },
        # funny (but valid) version string
        'openlmi-sw-test-funny-version-0:.1bA.+~-1%(disttag)s.noarch' : {
            'files' : {
                'usr/share/openlmi-sw-test-funny-version/' : {
                    'README'    : (REGULAR | DOC, 0644,  ''),
                }
            }
        },
        # funny (but valid) release string
        'openlmi-sw-test-funny-release-0:1.2.3-+.rel%(disttag)s.noarch' : {
            'files' : {
                'usr/share/openlmi-sw-test-funny-release/' : {
                    'README'    : (REGULAR | DOC, 0644,  ''),
                }
            }
        },
    },
}

SPEC_TEMPLATE = """\
Name:     %(pkg_name)s
Epoch:    %(epoch)s
Version:  %(version)s
Release:  %(release)s
License:  LGPLv2+
Source0:  %(pkg_name)s-%(epoch)s:%(version)s-%(release)s.%(pkg_arch)s.tar.gz
%(arch_string)s
Requires: openlmi-software
%(requires_string)s
Summary: This is a test package %(pkg_name)s

%%description
Test package for OpenLMI Software provider named %(pkg_name)s.

%%prep
%%setup -q

%%build
# no build necessary

%%install
%(install_string)s

%%files
%(files_string)s

%%changelog
* Mon Nov 04 2013 Michal Minar <miminar@redhat.com> 0.4.0-2
- initial commit
"""

REPO_CONFIG_TEMPLATE = """\
[%(repoid)s]
name=%(name)s
baseurl=file://%(repos_dir)s/%(repoid)s
gpgcheck=0
enabled=1
"""

def full_repo_name(repoid):
    """
    Convenient function returning full repository name in case it's in
    shortened form. Full repository name starts with ``openlmi-sw-test-repo-``.

    :param string repoid: Repository id either in shortened or full form.
        Must be present in ``REPOS``.
    :returns: Full repository name.
    :rtype: string
    """
    if not repoid in REPOS:
        if not repoid.startswith('openlmi-sw-test-repo-'):
            repoid = 'openlmi-sw-test-repo-' + repoid
    if not repoid in REPOS:
        raise ValueError('invalid repoid: %s' % repoid)
    return repoid

def full_pkg_name(pkg_name):
    """
    Convenient function returning full package name in case it's in
    shortened form. Full package name starts with ``openlmi-sw-test-``.

    :param string pkg_name: Package name either in shortened or full form.
        Must be present in one of testing repositories.
    :returns: full_pkg_name
    :rtype: string
    """
    if not pkg_name.startswith('openlmi-sw-test-'):
        pkg_name = 'openlmi-sw-test-' + pkg_name
    return pkg_name

def _unwind_files(path, files_dict):
    """
    Turns package files dictionary into a list of installed files.
    Called recursively until all entries from *files_dict* are handled.

    :param string path: Path where files are nested. If called the first time,
        it should be ``'/'``.
    :param dictionary files_dict: One of files dictionary from
        ``PKG_FILE_ENTRIES`` or one with the same format.
    :returns: List of absolute file paths of all package entries.
    :rtype: list
    """
    result = []
    for name, entry in files_dict.items():
        full_path = os.path.join(path, name)
        if isinstance(entry, dict):
            if name.endswith('/'):
                result.append(full_path[:-1])
            result.extend(_unwind_files(full_path, entry))
        else:
            result.append(full_path)
    return result

def _make_package_object(repo, pkg_nevra, rpm_path, pkg_dict):
    """
    Create package object with supplied informations. RPM package needs
    to be created first.

    :param repo: Instance of repository.
    :param string pkg_nevra: NEVRA of package to create.
    :param string rpm_path: Absolute path to rpm file.
    :param dictionary pkg_dict: One of ``REPOS`` values.
    :returns: Package object.
    :rtype: :py:class:`package.Package`
    """
    match = util.RE_NEVRA.match(pkg_nevra)
    pkg = package.Package(match.group('name'),
            epoch=match.group('epoch'),
            ver=match.group('ver'),
            rel=match.group('rel'),
            arch=match.group('arch'),
            repoid=repo.repoid,
            rpm_path=rpm_path,
            files=_unwind_files('/', pkg_dict.get('files', {})))
    return pkg

def _write_package_directory(path, entries=None):
    """
    Write contents of package to some directory. This content may then be
    archived into a source tarball.

    :param string path: Path to directory where the package contents shall
        be written.
    :param dictionary entries: File dictionary.
    """
    if entries is None:
        entries = {}
    os.makedirs(path)
    if path.endswith('perms/'):
        os.chmod(path,  0777 | stat.S_ISVTX)
    for name, entry in entries.items():
        full_path = os.path.join(path, name)
        if isinstance(entry, dict):
            _write_package_directory(full_path, entry)
        elif isinstance(entry, tuple):
            (ftype, perms, content) = entry
            if ftype & REGULAR:
                with open(full_path, 'w') as outf:
                    outf.write(content)
            elif ftype & LINK:
                os.symlink(content, full_path)
            elif ftype & (CHARDEV | BLOCKDEV):
                mode = ( (stat.S_IFCHR if ftype & CHARDEV else stat.S_IFBLK)
                       | perms)
                os.mknod(full_path, mode, os.makedev(content[0], content[1]))
            elif ftype & FIFO:
                os.mkfifo(full_path, perms)
            else:
                raise ValueError('file type 0%o unknown for file "%s"'
                        % (ftype, full_path))
            if not ftype & (CHARDEV | BLOCKDEV | FIFO | LINK):
                os.chmod(full_path, perms)
        else:
            raise TypeError('enexpected entry in package dictionary: "%s"'
                    % repr(entry))

def _make_source_tarball(rpmbuild_dir, pkg_nevra, pkg_dict):
    """
    Create a source tarball for rpm.

    :param string rpmbuild_dir: Path to directory where ``rpmbuild`` shall
        operate.
    :param string pkg_nevra: NEVRA string of package to build.
    :param dictionary pkg_dict: One of values of ``REPOS`` dictionary.
    :returns: Absolute file path to created tarball.
    :rtype: string
    """
    sources_dir = os.path.join(rpmbuild_dir, 'SOURCES')
    if not os.path.exists(sources_dir):
        os.mkdir(sources_dir)
    match = util.RE_NEVRA.match(pkg_nevra)
    tarball_path = os.path.join(sources_dir, pkg_nevra + '.tar.gz')
    archive_dir = tempfile.mkdtemp()
    try:
        src_dir = os.path.join(archive_dir,
                '%s-%s' % (match.group('name'), match.group('ver')))
        _write_package_directory(src_dir, pkg_dict.get('files', {}))
        subprocess.call(['/usr/bin/tar', '-C', archive_dir,
            '-czf', tarball_path, os.path.basename(src_dir)])
        return tarball_path
    finally:
        shutil.rmtree(archive_dir)

def _make_spec_files_string(path, entries):
    """
    Make a string used in spec file template to define a list of files
    belonging to package. Called recursively.

    :param string path: Absolute path to an entry to be added.
    :param dictionary entries: Dictionary representing directory content to
        include in resulting list.
    :returns: List of lines. Each representing one file or directory.
    :rtype: list
    """
    flag_to_prefix = {
            DOC : '%doc ',
            CONFIG : '%config(noreplace) ',
            0 : ''
    }
    res = []
    for name, entry in entries.items():
        full_path = os.path.join(path, name)
        if isinstance(entry, dict):
            if name.endswith('perms/'):
                res.append('%dir %attr(1777, -, -) ' + full_path)
            elif name.endswith('/'):
                res.append('%dir ' + full_path)
            res.extend(_make_spec_files_string(full_path, entry))
        elif isinstance(entry, tuple):
            if REGULAR & entry[0]:
                attrs = '%%attr(%o, -, -) ' % entry[1]
            else:
                attrs = ''
            res.append(flag_to_prefix[entry[0] & (DOC | CONFIG)]
                    + attrs + full_path)
        else:
            raise TypeError('enexpected entry in package dictionary: "%s"'
                    % repr(entry))
    return res

def _get_rpm_name_from_spec(spec_file_path):
    """
    :param string spec_file_path: Absolute file path to spec file.
    :returns: Name of rpm file builded out of particular spec file.
    :rtype: string
    """
    return subprocess.check_output(['/usr/bin/rpm', '-q', '--specfile',
        spec_file_path]).splitlines()[0] + '.rpm'

def _build_pkg(rpmbuild_dir, pkg_nevra, pkg_dict):
    """
    Build RPM package.

    :param string rpmbuild_dir: Absolute path to directory where
        the build takes place.
    :param string pkg_nevra: Nevra string of package to build.
    :param dictionary pkg_dict: One of ``REPOS`` values.
    :returns: Absolute file path to created rpm.
    :rtype: string
    """
    config = get_yum_config()
    specs_dir = os.path.join(rpmbuild_dir, 'SPECS')
    build_dir = os.path.join(rpmbuild_dir, 'BUILD')
    rpm_dir = os.path.join(rpmbuild_dir, 'RPMS')
    if not os.path.exists(specs_dir):
        os.mkdir(specs_dir)
    if not os.path.exists(build_dir):
        os.mkdir(build_dir)
    spec_path = os.path.join(specs_dir, pkg_nevra + '.spec')
    match = util.RE_NEVRA.match(pkg_nevra)
    if not match:
        raise ValueError('pkg_nevra is not a valid nevra: "%s"' % pkg_nevra)
    with open(spec_path, 'w') as spec:
        spec_args = {
            "pkg_name" : match.group('name'),
            "epoch"    : match.group('epoch'),
            "version"  : match.group('ver'),
            "release"  : match.group('rel'),
            "pkg_arch" : match.group('arch')
        }
        if match.group('arch') != get_yum_config().basearch:
            spec_args['arch_string'] = 'BuildArch: %s\n' % match.group('arch')
        else:
            spec_args['arch_string'] = ''
        for deps in ('requires', 'conflicts'):
            if deps in pkg_dict and pkg_dict[deps]:
                spec_args[deps + '_string'] = "\n".join(
                        ('%s: %s' % (deps.capitalize(), r))
                    for r in pkg_dict[deps]) % config._asdict()
            else:
                spec_args[deps + '_string'] = ''
        spec_args['files_string'] = "\n".join(
            _make_spec_files_string('/', pkg_dict.get('files', {})))
        if pkg_dict.get('files', []):
            spec_args['install_string'] = \
                    'mkdir -p ${RPM_BUILD_ROOT}\ncp -a ./* ${RPM_BUILD_ROOT}/'
        else:
            spec_args['install_string'] = ''
        spec.write(SPEC_TEMPLATE % spec_args)

    subprocess.call(['/usr/bin/rpmbuild', '--quiet',
        '-ba',                              # build rpm package
        '-D', '_topdir %s' % rpmbuild_dir,  # build in rpmbuild_dir
        spec_path                           # from this spec
        ])
    return os.path.join(rpm_dir, match.group('arch'),
        _get_rpm_name_from_spec(spec_path))

def _make_pkg(rpmbuild_dir, pkg_nevra, pkg_dict):
    """
    :returns: Absolute path to created rpm package.
    :rtype: string
    """
    _make_source_tarball(rpmbuild_dir, pkg_nevra, pkg_dict)
    return _build_pkg(rpmbuild_dir, pkg_nevra, pkg_dict)

def _create_repo(repos_dir, repoid, packages):
    """
    :returns: Absolute path to repository configuration file.
    :rtype: string
    """
    config_path = os.path.join('/etc/yum.repos.d', repoid + '.repo')
    repo_dir = os.path.join(repos_dir, repoid)
    with open(config_path, 'w') as config_file:
        config_file.write(REPO_CONFIG_TEMPLATE % {
            'repoid'    : repoid,
            'repos_dir' : repos_dir,
            'name'      : 'OpenLMI Software Test Repository - ' + ' '.join(
                        r.capitalize()
                    for r in repoid[len('openlmi-sw-test-repo-'):].split('-'))
        })
    os.makedirs(repo_dir)
    for rpm_path in packages:
        shutil.move(rpm_path, repo_dir)
    subprocess.call(['/usr/bin/createrepo', '--quiet', repo_dir])
    return config_path

def get_yum_config():
    """
    :returns: Yum configuration variables for this system.
    :rtype: :py:class:`YumVars`
    """
    if not hasattr(get_yum_config, '_yum_vars'):
        yumvar_dict = yum.YumBase().conf.yumvar
        kwargs = {   f: yumvar_dict[f]
                 for f in YumVars._fields if f in yumvar_dict}
        kwargs['disttag'] = '.' + subprocess.check_output(
                ['/usr/bin/rpm', '-q', '--qf', '%{RELEASE}\n'
                , 'kernel']).splitlines()[0].split('.')[-1]
        get_yum_config._yum_vars = YumVars(**kwargs)
    return get_yum_config._yum_vars

def make_repositories(repos_dir):
    """
    :returns: ``{ repoid : (config_path, rpms), ... }``
        Where rpms is a list of pairs:
            ``[ (pkg_nevra, rpm_path), ... ]``
    :rtype: dictionary
    """
    rpmbuild_dir = tempfile.mkdtemp()
    config = get_yum_config()
    try:
        # (repoid, (config_file_path, rpm_package_list))
        result = {}
        for repoid, packages in REPOS.items():
            rpms = []
            for pkg_tmpl, pkg_dict in packages.items():
                pkg_nevra = pkg_tmpl % config._asdict()
                rpms.append(
                        ( pkg_nevra
                        , _make_pkg(rpmbuild_dir, pkg_nevra, pkg_dict)))
            result[repoid] = (
                    _create_repo(repos_dir, repoid, [r for _, r in rpms]),
                    rpms)
    finally:
        shutil.rmtree(rpmbuild_dir)
    return result

def make_object_database(repos_dir):
    """
    :returns: ``(test_repo_db, other_repo_db)`` where
        both items have the following structure: ``{ repoid : repo, ... }``.
        First item contains repositories that shall be used for testing.
        Others are currently present on system and will be disabled during
        tests.
    :rtype: tuple
    """
    repos = make_repositories(repos_dir)
    db = dict((r.repoid, r) for r in repository.get_repo_database())
    result = {}
    for repoid, (_config_path, rpms) in repos.items():
        repo = db.pop(repoid)
        for pkg_nevra, rpm_path in rpms:
            pkg_name = util.RE_NEVRA.match(pkg_nevra).group('name')
            pkg_dict = [  d for p, d in REPOS[repoid].items()
                       if p.startswith(pkg_name)][0]
            pkg = _make_package_object(repo, pkg_nevra,
                    os.path.join(repo.local_path, os.path.basename(rpm_path)),
                    pkg_dict)
            repo.packages.add(pkg)
            result[repoid] = repo
    return (result, db)

class ObjectEncoder(json.JSONEncoder):
    """
    Takes care of encoding repository and package objects for json to handle
    library to handle.
    """
    def default(self, obj):
        if isinstance(obj, repository.Repository):
            return repository.to_json(self, obj)
        elif isinstance(obj, package.Package):
            return package.to_json(self, obj)
        elif isinstance(obj, datetime.datetime):
            return int(time.mktime(obj.timetuple()))
        elif isinstance(obj, set):
            return list(obj)
        return json.JSONEncoder.default(self, obj)

def save_test_database(repos_dir, test_repo_db, other_repo_db, dest_path=None):
    """
    Writes package database into a file.

    :param string repos_dir: Directory were repositories are stored.
    :param dictionary test_repo_db: Dictionary with testing repositories.
        It's the first item returned from :py:func:`make_object_database`.
    :param dictionary other_repo_db: Dictionary with other repositories.
        It's the second item returned from :py:func:`make_object_database`.
    :param string dest_path: Absolute file path, where the given objects will be
        serialized. If ``None``, some random, temporary file will be created.
    :returns: File path of written data.
    :rtype: string
    """
    if dest_path is None:
        dest_path = tempfile.mkstemp()
        db_file = os.fdopen(dest_path[0], 'w')
        dest_path = dest_path[1]
    else:
        db_file = open(dest_path, 'w')
    try:
        data = (repos_dir, test_repo_db, other_repo_db)
        json.dump(data, db_file, cls=ObjectEncoder,
                sort_keys=True, indent=4, separators=(',', ': '))
    finally:
        db_file.close()
    return dest_path

def load_test_database(src_path):
    """
    This is inverse function to :py:func:`save_pkg_database`.

    :returns: ``(repos_dir, test_repo_db, other_repo_db)`` package lists loaded
        from file.
    :rtype: tuple
    """
    with open(src_path, 'r') as db_file:
        repos_dir, test_db, other_db = json.load(
                db_file,
                object_hook=functools.partial(
                    repository.from_json, package.from_json))
    return repos_dir, test_db, other_db

def main():
    repos_dir = tempfile.mkdtemp()
    print "created repos dir: %s" % repos_dir
    db = make_object_database(repos_dir)
    import pprint
    pprint.pprint(db)

if __name__ == '__main__':
    main()

